@@ -5,7 +5,7 @@ use Config;
 use Exporter 'import';
 use Panda::Install::Payload;
 
-our $VERSION = '1.0.0';
+our $VERSION = '1.0.1';
 
 =head1 NAME
 
@@ -26,6 +26,7 @@ my $xsi_mask = '*.xsi';
 my $c_mask   = '*.c *.cc *.cpp *.cxx';
 my $h_mask   = '*.h *.hh *.hpp *.hxx';
 my $map_mask = '*.map';
+my $win32    = $^O eq 'MSWin32';
 
 sub write_makefile {
     _require_makemaker();
@@ -116,14 +117,17 @@ sub makemaker_args {
     
     # inject Panda::Install::ParseXS into xsubpp
     $postamble->{xsubpprun} = 'XSUBPPRUN = $(PERLRUN) -MPanda::Install::ParseXS $(XSUBPP)';
-
+    $params{LDFROM} ||= '$(OBJECT)';
+    
     delete $params{$_} for qw/SRC/;
     $params{OBJECT} = '$(O_FILES)' unless defined $params{OBJECT};
     
+    process_WIN32(\%params) if $win32;
+    
     delete $params{MODULE_INFO};
 
     $params{CCFLAGS} = "$Config{ccflags} $params{CCFLAGS}";
-
+    
     return %params;
 }
 
@@ -248,6 +252,8 @@ sub _apply_BIN_DEPS {
     $params->{CONFIGURE_REQUIRES}{$module}  ||= $installed_version;
     $params->{PREREQ_PM}{$module}           ||= $installed_version;
     $params->{MODULE_INFO}{BIN_DEPS}{$module} = $installed_version;
+    
+    _apply_BIN_DEPS_WIN32($params, $module) if $win32;
 
     my $info = Panda::Install::Payload::module_info($module);
     
@@ -299,6 +305,23 @@ sub _apply_BIN_DEPS {
     }
 }
 
+sub _apply_BIN_DEPS_WIN32 {
+    my ($params, $module) = @_;
+    
+    # add dll to linker list
+    my $list = $params->{MODULE_INFO}{WIN32}{DEPS_DLLS} ||= [];
+    my $module_path = $module;
+    $module_path =~ s#::#/#g;
+    die "SHOULDN'T EVER HAPPEN" unless $module =~ /([^:]+)$/;
+    my $last = $1;
+    foreach my $dir (@INC) {
+        my $dll_path = "$dir/auto/$module_path/$last.$Config{dlext}";
+        next unless -f $dll_path;
+        push @$list, $dll_path;
+        last;
+    }
+}
+
 sub process_BIN_SHARE {
     my $params = shift;
     my $bin_share = delete $params->{BIN_SHARE} or return;
@@ -335,6 +358,13 @@ sub process_BIN_SHARE {
     $pm->{$infopath} = '$(INST_ARCHLIB)/$(FULLEXT).x/info';
 }
 
+sub process_WIN32 {
+    my $params = shift;
+    my $deps_dlls = $params->{MODULE_INFO}{WIN32}{DEPS_DLLS};
+    return unless $deps_dlls and @$deps_dlls;
+    $params->{LDFROM} .= ' '.join(' ', @$deps_dlls);
+}
+
 sub attach_BIN_DEPENDENT {
     my $params = shift;
     my @deps = keys %{$params->{MODULE_INFO}{BIN_DEPS} || {}};
@@ -490,6 +520,7 @@ sub _string_merge {
 {
     package
         MYSOURCE;
+        
     sub postamble {
         my $self = shift;
         my %args = @_;
@@ -497,6 +528,61 @@ sub _string_merge {
     }
 }
 
+{
+    package
+        MY;
+    use Config;
+
+    if ($^O eq 'MSWin32') {
+        my $gcc_compliant = $Config{cc} =~ /\b(gcc|clang)\b/i ? 1 : 0;
+        
+        *dynamic_lib = sub {
+            my ($self, %attribs) = @_;
+            my $code = $self->SUPER::dynamic_lib(%attribs);
+            
+            unless ($gcc_compliant) {
+                warn(
+                    "Panda::Install: to maintain UNIX-like shared library behaviour on windows (export all symbols by default), we need gcc-compliant linker. ".
+                    "Panda::Install-dependant modules should only be installed on perls with MinGW shell (like strawberry perl), or at least having gcc compiler. ".
+                    "I will continue, but this module's binary dependencies may not work."
+                );
+                return $code;
+            }
+            return $code unless $code;
+    
+            # remove .def-related from code, remove double DLL build, remove dll.exp from params, add export all symbols param.
+            my $DLLTOOL = $Config{dlltool} || 'dlltool';
+            my (@out, $last_ld);
+            map { $last_ld = $_ if /\$\(LD\)\s/ } split /\n/, $code;
+            foreach my $line (split /\n/, $code) {
+                next if $line =~ /$DLLTOOL/; # drop dlltool calls (we dont need .def file)
+                if ($line =~ /\$\(LD\)\s/) {
+                    next if $line ne $last_ld;
+                    $line =~ s/\$\(LD\)\s/\$(LD) -Wl,--export-all-symbols /;
+                    $line =~ s/\bdll\.exp\b//;
+                }
+                $line =~ s/\$\(EXPORT_LIST\)//g; # remove <PACKAGE>.def from target dependency
+                push @out, $line;
+            }
+            
+            $code = join("\n", @out);
+            return $code;
+        };
+        
+        *dlsyms = sub {
+            my ($self, %attribs) = @_;
+            return '' if $gcc_compliant; # our dynamic_lib target doesn't need any .def files with gcc
+            return $self->SUPER::dlsyms(%attribs);
+        };
+    }
+}
+
+
+#    dlsyms  
+    
+    # generate DLL file containing all symbols, like default behaviour on UNIX.
+
+
 sub _require_makemaker {
     unless ($INC{'ExtUtils/MakeMaker.pm'}) {
         require ExtUtils::MakeMaker;
