@@ -291,6 +291,7 @@ sub print_section {
                     my $tmfunc_code = _typemap_inline_func($tmfunc_name, $class, $code, $arg_init, $is_input, \@args, $tmap->{_init_code});
                     $gen_funcs{$tmfunc_name} = $tmfunc_code;
                 }
+                $expr =~ s/^\s*\(\s*/(aTHX_ /; # add interp to param for threaded perls
                 $line =~ s/$re/${tmfunc_name}::get$expr/;
             }
         }
@@ -323,13 +324,13 @@ sub _typemap_inline_func {
     
     $code =~ s/^/        /mg;
     return << "EOF" if $is_input;
-        struct $tmfunc_name { static inline $class get (SV* arg$additional_args) {
+        struct $tmfunc_name { static inline $class get (pTHX_ SV* arg$additional_args) {
             $class var;\n    $code;
             return var;
         }};
 EOF
     return << "EOF";
-        struct $tmfunc_name { static inline SV* get ($class var$additional_args) {
+        struct $tmfunc_name { static inline SV* get (pTHX_ $class var$additional_args) {
             SV* arg$arg_init;\n    $code;
             return arg;
         }};
